<?php
/*
Plugin Name: Dita XHTML Help Import (wpmu version)
Plugin URI: http://zed1.com/journalized/wordpress-plugins/
Description: This file lets you Import WordPress pages from a DITA help XHTML hierarchy.
Author: Mike Little
Version: 1.0.0
Author URI: http://zed1.com/
License: GNU GPL

Copyright 2008 Northgate Information Systems

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

class Ditahelp_Import {
    var $HELP_IMPORT_VERSION = 'ditahelp_importer_version';
    var $HELP_IMPORT_LASTDIR = 'ditahelp_importer_lastdir';
    var $HELP_IMPORT_DELFIRST = 'ditahelp_importer_delfirst';
    var $HELP_IMPORT_WHICH_BLOG = 'ditahelp_importer_whichblog';
    var $HELP_ROOT_PATH = '/help';  // this shold be derived from the current path or the WPMU blog selected.
    var $table_name = '';
    var $table_def = '';
    var $version = '97'; // internal database version
    var $importdirectory = '';
    var $posts = array ();
    var $file;
	var $whichblog = -1;
    var $debug = true;
    var $debug_extra = false;
    var $delfirst = 0;
    var $delfromstaging = 0;
    var $loop_size = 75;
	var $HELP_IMPORT_POST_TYPE = 'page';
    var $is_wpmu = false; // auto detected
    var $id_must_match_filename = false; //true; (internal requirement)

    function header() {
        echo '<div class="wrap">';
        echo '<h2>'.__('Import DITA Help (version '). $this-> version  . ')</h2>';
    } // end header

    function footer() {
        echo '</div>';
    } // end footer

    function greet() {
        echo '<div class="narrow">';
        echo '<p>'.__('Howdy! This importer allows you to extract pages from a DITA help XHMTL hierarchy of pages into your blog. Pick a DITA help directory to upload and click Import.').'</p>';
        echo '<p>'.__('This may take a few minutes depending on the number of files. Please be patient.').'</p>';
        echo '<form action="admin.php?import=ditahelp&amp;step=1" method="post">';
        $checked = '';
        $this->delfirst = get_option($this-> HELP_IMPORT_DELFIRST);
		//if ($delfirst) $checked = ' checked="checked"'; // never preset the delete everything option
        //*
        printf('<p><label for="delfirst">%s</label> <input type="checkbox" name="delfirst" id="delfirst" value="1" %s /><br />', 'Do you want to clear all posts, comments, and ratings first?',  $checked);
        echo '<span style="font-size:80%">(This will truncate the following tables: wp_comments, wp_postmeta, wp_posts, and wp_ratings.)</span></p>';
        //*/
        printf('<p><label for="delfromstaging">%s</label> <input type="checkbox" name="delfromstaging" id="delfromstaging" value="1" %s /><br />', 'Do you want to clear all posts based on the contents of staging table?',  $checked);
        echo '<span style="font-size:80%">(Only use this after an import failed part way through.)</span></p>';

        if ($this->is_wpmu) {
            echo('<p><label for="whichblog">Application Area:</label> <select name="whichblog" id="whichblog" style="width:300px">' . "\n");
            $blog_list = get_blog_list( 0, 'all');
            usort($blog_list, _dhcmp);

            foreach ((array) $blog_list as $key => $details) {
                if (($details['path'][strlen($details['path']) - 1]) == '/')
                    $details['path'] = substr($details['path'], 0, -1);
                $name = get_blog_option( $details['blog_id'], "blogname" );
                echo('<option value="' . $details['blog_id'] . ':' . $details['path'] . '">' . $name . '</option>' . "\n");
            }
            echo('</select></p>');
        }

        printf('<p><label for="impdir">%s</label> <input type="text" name="impdir" id="impdir" style="width:450px" value="%s" />', __('DITA help Directory:'), get_option($this-> HELP_IMPORT_LASTDIR));
        echo '<br /><span style="font-size:80%">(Please use forward slashes (\'/\'), even on Windows.)</span></p>';

        echo '<p class="submit"><input type="submit" name="submit" value="'.__('Import Files').' &raquo;" /></p>';
        echo '</form>';
        echo '</div>';
    } // end greet

    function dispatch() {
		if (empty ($_GET['step']))
			$step = 0;
		else
			$step = (int) $_GET['step'];

		$this-> header();

		switch ($step) {
			case 0 :
				global $current_blog;
				if ($this->is_wpmu && $current_blog->blog_id != 1) {
					echo '<div class="narrow">';
					echo '<p class="error">'.__('Sorry, you need to be on the main <strong>Northgate Online Help</strong> tab to run this importer</p>');
					echo '</div>';
				} else {
					$this-> check_setup();
					$this-> greet();
				}
				break;
			case 1 :
				if ($_POST['impdir']) {
					$this->importdirectory = $_POST['impdir'];
                    $this->importdirectory = str_replace('\\', '/', $this->importdirectory);
					update_option($this->HELP_IMPORT_LASTDIR, $this->importdirectory);
				}
				if ($_POST['delfirst']) {
					$this->delfirst = true;
					update_option($this->HELP_IMPORT_DELFIRST, $this->delfirst);
				}
				if ($_POST['delfromstaging']) {
					$this->delfromstaging = true;
                }
                if ($this->is_wpmu) {
                    $this->whichblog = $_POST['whichblog'];
                    update_option($this->HELP_IMPORT_WHICH_BLOG, $this->whichblog);
                    list($this->whichblog, $this->HELP_ROOT_PATH) = split(':', $this->whichblog);
                } else {
                    $this->whichblog = 1;
                }

				$this-> import($step);
				break;
			case 2 :
                if ($this->is_wpmu) {
                    $this->whichblog = get_option($this->HELP_IMPORT_WHICH_BLOG);
                    list($this->whichblog, $this->HELP_ROOT_PATH) = split(':', $this->whichblog);
                } else {
                    $this->whichblog = 1;
                }
				$this-> import($step);
				break;
			case 3 :
                if ($this->is_wpmu) {
                    $this->whichblog = get_option($this->HELP_IMPORT_WHICH_BLOG);
                    list($this->whichblog, $this->HELP_ROOT_PATH) = split(':', $this->whichblog);
                } else {
                    $this->whichblog = 1;
                }
				$this-> import($step);
				break;
			case 4 :
                if ($this->is_wpmu) {
                    $this->whichblog = get_option($this->HELP_IMPORT_WHICH_BLOG);
                    list($this->whichblog, $this->HELP_ROOT_PATH) = split(':', $this->whichblog);
                } else {
                    $this->whichblog = 1;
                }
				$this-> import($step);
				break;
			case 5 :
                if ($this->is_wpmu) {
                    $this->whichblog = get_option($this->HELP_IMPORT_WHICH_BLOG);
                    list($this->whichblog, $this->HELP_ROOT_PATH) = split(':', $this->whichblog);
                } else {
                    $this->whichblog = 1;
                }
				$this-> import($step);
				break;
		}

		$this-> footer();
	} // end dispatch



	function import($stage = 1) {
		global $wpdb;

		echo '<div><ul>';
		/**

		Processing:

		1. Empty the staging table.
		2. Load all the pages into a temp table.
	    3. Compare Id's to find all the pages which are replacements for existing WP pages
		4. Compare content (title, body) of the pages to see if they have changed.
		5. Of the ones which have changed, flag the ones which have comments.
		6. Update the WP pages.
		7. Add in the new WP pages.
		7.2. Set parents of all pages
		8. Determine the removed pages (exist in WP not in temp table), add page_deleted keyword and flag.

		All done!
		*/


		switch ($stage) {
			case 1 :
				if ($this->delfromstaging) {
                    $this-> delete_posts_from_staging_table();
                }

				// 1. Empty the staging table.
				echo '<li>' . __('Clearing staging table.') .'</li>';
				$wpdb-> query("TRUNCATE TABLE " . $this-> table_name);
				if ($this-> delfirst) {
					echo '<li>' . __('Clearing other tables.') .'</li>';
					$wpdb-> query("DELETE $wpdb->term_relationships FROM $wpdb->term_taxonomy RIGHT JOIN $wpdb->term_relationships ON ($wpdb->term_taxonomy.term_taxonomy_id = $wpdb->term_relationships.term_taxonomy_id) WHERE $wpdb->term_taxonomy.taxonomy = 'category'");
					$wpdb-> query("TRUNCATE TABLE `wp_comments`");
					$wpdb-> query("TRUNCATE TABLE `wp_postmeta`");
					$wpdb-> query("TRUNCATE TABLE `wp_posts`");
					$wpdb-> query("TRUNCATE TABLE `wp_ratings`");
                }
                if ($this->is_wpmu) {
                    echo '<li>' .__('Switching to blog: ') . $this->whichblog . '</li>';
                    //echo '<li>' .__('HELP_ROOT_PATH = ') . $this->HELP_ROOT_PATH . '</li>';
                    switch_to_blog($this->whichblog);
                }
				// 2. Load all the pages into a temp table.
				echo '<li>' .__('Processing directory: ') . $this->importdirectory . '</li>';
				// Are we opening a directory?
				if (!is_dir($this-> importdirectory)) {
					echo '<li class="error">' . __('Error: ') . $this->importdirectory . __(' is not a directory or is not readable.') . '</li>';
					die();
				}
				$this-> load_files();

				echo '</ul>';
				echo '<form action="admin.php?import=ditahelp&amp;step=2" method="post">';
				echo '<p class="submit"><input type="submit" name="submit" value="'.__('Stage 2').' &raquo;" /></p>';
				echo '</form></div>';

				break;

			case 2:
				//3. Compare Id's to find all the pages which are replacements for existing WP pages
                if ($this->is_wpmu) {
                    echo '<li>' .__('Switching to blog: ') . $this->whichblog . '</li>';
                    //echo '<li>' .__('HELP_ROOT_PATH = ') . $this->HELP_ROOT_PATH . '</li>';
                    switch_to_blog($this->whichblog);
                }
				echo '<li>' . __('Marking updates.') .'</li>';
				$count = $this-> mark_updates();

                  if ($count) {
                      echo '<li>' . __('Processing updates.') .'</li>';
                      $this-> process_updates();
                      echo '</ul>';
                      echo '<form action="admin.php?import=ditahelp&amp;step=2" method="post">';
                      echo '<p class="submit"><input type="submit" name="submit" value="'.__('Next ').$this-> loop_size.' &raquo;" /></p>';
                      echo '</form></div>';
                  } else {
                      $this-> mark_new();
                      $this-> update_guids();
                      echo '</ul>';
                      echo '<form action="admin.php?import=ditahelp&amp;step=3" method="post">';
                      echo '<p class="submit"><input type="submit" name="submit" value="'.__('Stage 3').' &raquo;" /></p>';
                      echo '</form></div>';
                  }
				break;

			case 3:
				  // 7. Add in the new WP pages.
                if ($this->is_wpmu) {
                    echo '<li>' .__('Switching to blog: ') . $this->whichblog . '</li>';
                    //echo '<li>' .__('HELP_ROOT_PATH = ') . $this->HELP_ROOT_PATH . '</li>';
                    switch_to_blog($this->whichblog);
                }
				echo '<li>' . __('Collate and insert.') .'</li>';flush();
				  $count = $this-> get_new_posts();
                  if ($count) {
                      $this-> insert_pages();
                      echo '</ul>';
                      echo '<form action="admin.php?import=ditahelp&amp;step=3" method="post">';
                      echo '<p class="submit"><input type="submit" name="submit" value="'.__('Next ').$this-> loop_size.' &raquo;" /></p>';
                      echo '</form></div>';
                  } else {
                      echo '<li>' . __('Completed') .'</li>';flush();
                      global $wp_rewrite;
                      $wp_rewrite->flush_rules();
                      //$this-> update_guids();

                      echo '</ul>';
                      echo '<form action="admin.php?import=ditahelp&amp;step=4" method="post">';
                      echo '<p class="submit"><input type="submit" name="submit" value="'.__('Stage 4').' &raquo;" /></p>';
                      echo '</form></div>';
                  }
				  break;


			case 4:
                // sort parentage
                if ($this->is_wpmu) {
                    echo '<li>' .__('Switching to blog: ') . $this->whichblog . '</li>';
                    //echo '<li>' .__('HELP_ROOT_PATH = ') . $this->HELP_ROOT_PATH . '</li>';
                    switch_to_blog($this->whichblog);
                }
				echo '<li>' . __('Sorting parentage.') .'</li>';flush();
				$this-> sort_parentage();
                  global $wp_rewrite;
				  $wp_rewrite->flush_rules();
                $this-> update_guids();

				echo '</ul>';
				echo '<form action="admin.php?import=ditahelp&amp;step=5" method="post">';
				echo '<p class="submit"><input type="submit" name="submit" value="'.__('Stage 5').' &raquo;" /></p>';
				echo '</form></div>';
				break;

			case 5:
			case 6:
				echo '<h3>';
				printf(__('All done. <a href="%s">Have fun!</a>'), get_option('home'));
				echo '</h3>';
				break;
		} // end switch
	} // end import


	function check_setup() {
		if ($this-> check_table_exists()) {
			if(intval(get_option($this-> HELP_IMPORT_VERSION)) < intval($this-> version)) {
				echo '<p>' . __('Upgrading Importer...') . '</p>';
				$new_version = $this-> update();
			}
		} else {
			$new_version = $this-> version;
		}
		if ($new_version == $this-> version) {
			update_option($this-> HELP_IMPORT_VERSION, $this-> version);
		}
	} // end check_setup

	function check_table_exists() {
		global $wpdb;
		$tables = $wpdb-> get_col('SHOW TABLES;');
		foreach($tables as $table) {
			// If a row exists for the database table...
			if ($table == $this-> table_name) {
				$table_exists = true;
			}
		}
		if (!$table_exists) {
			echo '<p>' . __('This looks like the first time you have run this importer. We need to create the staging table...') . '</p>';
			$wpdb-> query($this-> table_def);
			echo '<p>' . __('...Done.') . '</p>';
		}
		return $table_exists; // false if newly created
	} // end check_table_exists

	function load_files() {
		$this-> dir = new RecursiveDirectoryIterator($this-> importdirectory);
		foreach (new RecursiveIteratorIterator($this-> dir) as $file) {
			if (preg_match('@\.(html)$@i',$file-> getFilename())) {
                $original_html_pathname = str_replace('\\', '/', $file-> getPathname());
				if ($this->debug) echo('<li>' .__('Importing file ') .  $original_html_pathname . "\n<ul>\n"); flush();
				$this-> import_file($file);
                if ($this->debug) echo("</ul></li>\n"); flush();
			}
		}
	} // end load_files

    function processFilenameToId($path) {
		if ($this->debug_extra) echo("<li>processFilenameToId got ". $path . "</li>\n"); flush();
        $theId = str_replace('.html', '', basename($path));
		if ($this->debug_extra) echo("<li>processFilenameToId id=$theId</li>\n"); flush();
        return $theId;
    } // end processFilenameToId

	function import_file($file) {
		global $wpdb;
		$body = '';
		$title = '';
		$id = '';

		$dom = new DomDocument();
		@$dom-> load($file);
        $dxp = new DOMXPath($dom);

        if ('index.html' == $file-> getFilename()) {
			return;
		}

        $original_html_pathname = str_replace('\\', '/', $file-> getPathname());
		$original_html_dirname = dirname($original_html_pathname);
		if ($this->debug_extra) echo("<li>pathname = $original_html_dirname</li>\n"); flush();
		$relative_filepath = preg_replace('#'.$this-> importdirectory.'#', '', $original_html_dirname);
		if ($this->debug_extra) echo("<li>relative_filepath=$relative_filepath</li>\n"); flush();

		// fix the paths
		if ($this->debug_extra) echo("<li>** Fixing links for $file </li>\n"); flush();
		$anchors = $dom-> getElementsByTagName('a');
		$empty_anchors_count = 0;
		$anchors_to_delete = null;
		foreach ($anchors as $a) {
			$href = $a-> getAttribute('href');
			if (!empty($href)) {
                $href = $this->HELP_ROOT_PATH . '/' . $this-> processFilenameToId($href); // hard coded -- BAD
                $a-> setAttribute('href', $href);
			} else {
				++$empty_anchors_count;
				if (!$a ->hasAttribute('href')) {
					$anchors_to_delete[] = $a;
				}
			}
		} // end for each anchor

		$c = count($anchors_to_delete);
		if ($c) {
			foreach ($anchors_to_delete as $a) {
				$a->parentNode-> removeChild($a);
			}
		}
		if ($empty_anchors_count) {
			if ($this->debug_extra) echo("<li>there were $empty_anchors_count empty anchors. $c were deleted </li>\n"); flush();
		}

		if ($this->debug_extra) echo("<li>... finished fixing links for $file </li>\n"); flush();

		// Look for the short description and turn it into the excerpt.

		// Get the metatag
		$metatags = $dom-> getElementsByTagName('meta');
		foreach ($metatags as $metatag) {
			$name = $metatag-> getAttribute('name');
			if ($name == 'description') {
				$description = $metatag-> getAttribute('content');
				if ($this->debug_extra) echo('<li> got meta tag description ' .  $description . "</li>\n"); flush();
				$excerpt = $description;
			} else if ($name == 'prodname') {
				$prodname = $metatag-> getAttribute('content');
				if ($this->debug_extra) echo('<li> got meta tag prodname ' .  $prodname . "</li>\n"); flush();
			} else if ($name == 'keywords') {
				$keywords  .= $metatag-> getAttribute('content');
				if ($this->debug_extra) echo('<li> got meta tag keywords ' .  $keywords . "</li>\n"); flush();
			} else if ($name == 'deleted') {
				$deleted_text = $metatag-> getAttribute('content');
				if ($this->debug) echo('<li> got meta tag deleted ' . $deleted_text . "</li>\n"); flush();
			} else if ($name == 'replaced-by') {
				$replaced_by = $metatag-> getAttribute('content');
				if ($this->debug) echo('<li> got meta tag replacedby ' . $replaced_by . "</li>\n"); flush();
			}
		} // end while


		// find all images
		// adjust paths
		// copy files from source to new path

		$allimages = $dom-> getElementsByTagName('img');
		foreach ($allimages as $img) {

			$src = $img-> getAttribute('src');
			if (!empty($src)) {
				if ($this->debug_extra) echo('<li>images src = ' .  $src . "</li>\n"); flush();
				$src = $original_html_dirname . '/' . $src; // add original source path to relative src path
				if ($this->debug_extra) echo('<li>images src = ' .  $src . "</li>\n"); flush();
				$src = getRealpath($src); // refine all ..'s
				if ($this->debug_extra) echo('<li>images realpath src = ' .  $src . "</li>\n"); flush();
				$src = str_replace('\\', '/', $src); // forward slash works on all platforms
				$original_source = $src; // remember this for the copy
				if (!empty($src)) {
					//now strip off to get a relative path
					$src = preg_replace('#'. $this-> importdirectory . '#', '', $src); // strip
					$src = str_replace('\\', '/', $src); // again? forward slash works on all platforms
					if ($this->debug_extra) echo('<li>images  src now = ' .  $src . "</li>\n"); flush();

                    if ($this->is_wpmu) {
                        $dest = constant('ABSPATH') . "wp-content/blogs.dir/{$wpdb->blogid}/files/"; // mu specific
                    } else {
                        $dest = ABSPATH; // non-mu specific
                    }
					if ($this->debug_extra) echo('<li>images 1 dest =' .  $dest . "</li>\n"); flush();
                    $dest = str_replace('\\', '/', $dest . $src); //again?
                    $dest = str_replace('//', '/', $dest);
					if ($this->debug_extra) echo('<li>images dest now = ' .  $dest . "</li>\n"); flush();

                    if ($this->is_wpmu) {
                        $src = $this->HELP_ROOT_PATH . '/files/' . $src; // mu specific
                    } else {
                        $src = $this->HELP_ROOT_PATH . $src; // non-mu specific
                    }
					$src = str_replace('//', '/', $src);

					$img-> setAttribute('src', $src);
					if ($this->debug_extra) echo('<li> put back ' .  $img-> getAttribute('src') . "</li>\n"); flush();

					// create directory if required
					if (!is_dir(dirname($dest))) {
						if ($this->debug) echo("<li> need to create directory " . dirname($dest) . "</li>\n"); flush();
						mkdirs(dirname($dest));
					}
					if (copy($original_source, $dest)) {
						if ($this->debug) echo("<li> copied $original_source to $dest</li>\n"); flush();
					} else {
						echo("<li class=\"error\"> failed to copy $original_source to $dest</li>\n"); flush();
					}

					// hack to fix known dita bug: alt attributes have literal \n prepended to them which then gets encoded as &#10;
					$alt = $img-> getAttribute('alt');
					str_replace('&#10;', '', $alt);
					$img-> setAttribute('alt', $alt);

				} else {
					echo('<li class="error"> error could not resolve src ' . $src . "</li>\n"); flush();
				}
			} // end if not empty
		} // end for all images

		// remove short description inserted for related topics.
		// look for li with class ulchildlink (child of ul class illinks)
		// format is <strong><a>link text</a></strong>short desc</li>
		$lis = $dom-> getElementsByTagName('li');
		foreach ($lis as $li) {
			$classes = $li -> getAttribute('class');
			if ($classes && strstr($classes, 'ulchildlink')) {
				// get first child (strong containing link)
				$strong = $li ->firstChild ->cloneNode(true);
				// remove children
				while ($li-> hasChildNodes()) {
					$li -> removeChild($li ->firstChild);
				}
				// put it back
				$li ->appendChild($strong);
			}
		}

		// find span for related topics and turn it into an h3.
		$spans = $dom-> getElementsByTagName('span');
		foreach ($spans as $span) {
			$classes = $span -> getAttribute('class');
			if ($classes && strstr($classes, 'related-topics')) {
				// get first child (containing text)
				$txt = $span ->firstChild ->cloneNode(true);
				// get parent and remove it!
				$grandparent_div = $span ->parentNode->parentNode;
				// remove children
				while ($grandparent_div ->hasChildNodes()) {
					$grandparent_div -> removeChild($grandparent_div ->firstChild);
				}
				// put it back as an h3
				$h3 = $dom->createElement('h3');
				$h3 -> setAttribute('class', 'related-topics');
				$h3 -> appendChild($txt);
				$grandparent_div-> appendChild($h3);
			}
		}

        // find parent and store as post_parent_name
        // format is <div class="parentlink"><strong>Parent topic:</strong> <a href="topic id">topicname</a></div>
        $parents = $dxp-> query("//div[attribute::class='parentlink']/a");
        if (count($parents) > 0) {
            if (count($parents) > 1) {
                echo('<li class="error">Warning: more than one parent. Using first ' . "</li>\n"); flush();
            }
            foreach ($parents as $par) {
                $href = $par-> getAttribute('href');
                $parent_id = $this-> processFilenameToId($href);
                if ($this->debug) echo("<li> parent id of ".str_replace('.html', '', basename($file-> getFilename()))." is $parent_id</li>\n"); flush();
                break;
            }
        }



		// get the body, convert to text, strip the start/end body tags.
		$bodynodes = $dom-> getElementsByTagName('body');
		foreach ($bodynodes as $b) {
			$id = trim($b-> getAttribute('id'));
			$bodycontent = $dom-> saveXML($b);
			$res = preg_match('#<body[^>]*>(.*)</body>#s', $bodycontent, $matches);
			$body .= trim($matches[1]);
		}

		// we need to check the id matches the trimmed filename
        $required_id = $this-> processFilenameToId($file-> getFilename());
		if ($this->id_must_match_filename && ($id !== $required_id)) {
			echo('<li class="error">Id (' . $id . ') does not match filename for file ' .  $file-> getPathname() . "</li>\n"); flush();
			return;
		}
        $id = $required_id; // they either match or they don't need to match

		// get the title
		$titles = $dom-> getElementsByTagName('title');
		foreach ($titles as $t) {
			$title = trim($t-> textContent);
		}

		// insert gathered data
		if (!empty($id) && !empty($body)) {
			$relative_filepath = ltrim($relative_filepath, '/');
			// insert contents into db table
			$deleted_flag = "N";
			if (!empty($deleted_text)) $deleted_flag = "Y";
			$query = "INSERT INTO " . $this-> table_name .
                     " (`ID`," .
                     " `post_author`, " .
                     " `post_date`," .
                     " `post_content`," .
                     " `post_title`, " .
                     " `post_category`, " .
                     " `post_excerpt`, " .
                     " `post_keywords`, " .
                     " `post_name`, " .
                     " `post_parent`, " .
                     " `post_parent_name`, " .
                     " `post_parent_tab`, " .
                     " `post_prodname`, " .
                     " `post_path`, " .
                     " `menu_order`, " .
                     " `is_path`, " .
                     " `is_deleted`, " .
                     " `has_comments`, " .
                     " `deleted_text`, " .
                     " `replaced_by`, " .
                     " `is_new`) ";
			$query .= "\n VALUES(" .
                     " 0," .                                            //`ID`," .
                     " 0, " .                                           //`post_author`, " .
                     " '" . current_time('mysql', 1). "'," .            //`post_date`," .
                     " '" . $wpdb-> escape($body) . "', " .             //`post_content`," .
                     " '" . $wpdb-> escape($title) . "'," .             //`post_title`, " .
                     " 0," .                                            //`post_category`, " .
                     " '" . $wpdb-> escape($excerpt) . "'," .           //`post_excerpt`, " .
                     " '" . $wpdb-> escape($keywords) . "', " .         //`post_keywords`, " .
                     " '" . $wpdb-> escape($id) . "'," .                //`post_name`, " .
                     " '0'," .                                          //`post_parent`, " .
                     " '". $parent_id ."'," .                           //`post_parent_name`, " .
                     " ''," .                                           //`post_parent_tab`, " .
                     " '" . $wpdb-> escape($prodname) . "'," .          //`post_prodname`, " .
                     " '" . $wpdb-> escape($relative_filepath) . "'," . //`post_path`, " .
                     " 0," .                                            //`menu_order`, " .
                     " 'N'," .                                          //`is_path`, " .
                     " '$deleted_flag'," .								//`is_deleted`, " .
                     " 'N'," .                                          //`has_comments`, " .
                     " '" . $wpdb-> escape($deleted_text) . "'," .		//`deleted_text`, " .
                     " '" . $wpdb-> escape($replaced_by) . "'," .		//`replaced_by`, " .
                     " 'N')";                                           //`is_new`) ";
			//if ($this->debug_extra) echo("query=<pre>[\n$query\n]</pre>\n");
            $result = $wpdb-> query($query);
            if ($wpdb-> rows_affected != 1) {
                echo('<li class="error">inserting file failed' . $result . "</li>\n"); flush();
            }
            //echo('<li>... imported. ' . $wpdb-> rows_affected . "</li>\n"); flush();
		}
	} // end import_file



	function mark_updates () {
		global $wpdb;
		// select from staging table all page_names (slugs)
		// for each one, check whether that slug already exists,
		// if it does set the id to the post id

		$query = "SELECT $this->table_name.post_name AS post_name, $wpdb->posts.ID AS ID," .
				 " $this->table_name.post_content AS post_contentA, $wpdb->posts.post_content AS post_contentB," .
				 " $this->table_name.post_title AS post_titleA, $wpdb->posts.post_title AS post_titleB," .
				 " $wpdb->posts.comment_count" .
				 " FROM $this->table_name " .
                 "  RIGHT JOIN $wpdb->posts ON $this->table_name.post_name = $wpdb->posts.post_name " .
                 " WHERE is_path='N' " .
                 " LIMIT $this->loop_size";
		if ($this->debug_extra) echo("<li>new sql $query</li>\n");

		$rows = $wpdb-> get_results($query, ARRAY_A);
        $count = 0;
		if ($rows != null) {
			foreach ($rows as $row) {
				$post_name =  $wpdb-> escape($row['post_name']);
				$page_id = $row['ID'];
                if ($post_name && $page_id) {
                    if ($this->debug) echo("<li>Found existing topic with slug: $post_name, id=$page_id</li>\n");

					$wpdb-> query("UPDATE " . $this-> table_name . " SET ID=" . intval($page_id) . ", is_updated='Y' WHERE post_name='" . $post_name . "'");
					++$count;
					// 4. Compare content (title, body) of the pages
					// to see if they have changed.
					if (($wpdb-> escape($row['post_contentA']) != $wpdb-> escape($row['post_contentB'])) ||
						  ($wpdb-> escape($row['post_titleA']) != $wpdb-> escape($row['post_titleB']))) {
						if ($this->debug) echo("<li>Title or Content differ: $post_name, id=$page_id</li>\n");
						if ($row['comment_count']) {
							$wpdb-> query("UPDATE " . $this-> table_name . " SET ID=" . intval($page_id) . ", is_updated='U' WHERE post_name='" . $post_name . "'");
						} else {
							if ($this->debug) echo("<li>...but no comments assigned.</li>\n");
						}
					} // end if content updated
				} // end if found one
            } // end for each

        } // end if any rows
        return $count;
	} // end mark_updates


    function mark_new() {
		global $wpdb;
        // set the rest to new
        $res = $wpdb-> query("UPDATE " . $this-> table_name . " SET is_new='Y' WHERE ID=0 && is_updated='N'");
        $rval = $wpdb-> rows_affected;
        if ($this->debug) echo("<li>$rval pages were new </li>\n");
    }

    function process_updates() {
		global $wpdb;
		// Select from staging table all pages which are marked for update.
        // extract the data and call update.
		// Remember the onese with content changes (is_updated == U) and create
		// a post about them.
		$post_updates = array();
        $query = "SELECT * " .
                 " FROM $this->table_name " .
                 " WHERE (is_updated='Y' OR  is_updated='U') AND is_path='N'";

		$rows = $wpdb-> get_results($query, ARRAY_A);
		if ($rows != null) {
			foreach ($rows as $row) {

                if ($this->debug) echo "<li>".__('Updating post ') . $row['post_name'] . " id = " . $row['ID'] . "</li>\n"; flush();
				$ID = $row['ID'];
				$post_author = 1;
				$post_date_gmt = $wpdb-> escape($row['post_date']);
				$post_date = get_date_from_gmt($post_date_gmt);
				$post_content = $wpdb-> escape($row['post_content']);
				$post_title = $wpdb-> escape($row['post_title']);
				$post_name = $wpdb-> escape($row['post_name']);
				$post_path = $wpdb-> escape($row['post_path']);
				$post_excerpt = $wpdb-> escape($row['post_excerpt']);
                $post_parent =  $wpdb-> escape($row['post_parent']);
                $post_parent_tab =  $wpdb-> escape($row['post_parent_tab']);
				$post_prodname = $wpdb-> escape($row['post_prodname']);
				$deleted_text = $wpdb-> escape($row['deleted_text']);
				$replaced_by = $wpdb-> escape($row['replaced_by']);
				$post_status = 'publish';
				$post_type = $this->HELP_IMPORT_POST_TYPE;
                $postdata = compact('ID', 'post_author', 'post_date_gmt', 'post_date',
									'post_content', 'post_title', 'post_name', 'post_path',
									'post_excerpt', 'post_parent', 'post_parent_tab',
									'post_prodname', 'deleted_text', 'replaced_by', 'post_status',
									'post_type');

                $res = wp_update_post($postdata);
                if ($this->debug_extra) echo "<li>".__('Updated post ') . $row['post_name'] . " response is $res</li>\n"; flush();
                $wpdb-> query("UPDATE " . $this-> table_name . " SET is_path='Y' WHERE ID =$ID");
				if ($row['is_updated'] == 'U') {
					// Add it to the list
					$post_updates[] = array( 'title' => $post_title, 'slug' => $post_name );
				}
				list($prodname_id, $prodname_long) = split(':', $post_prodname);
                add_post_meta($post_id, 'prodname_id', $prodname_id, true);
                add_post_meta($post_id, 'prodname_long', $prodname_long, true);
				if (!empty($deleted_text)) {
					if ($this->debug) echo("<li> topic is deleted: $deleted_text</li>\n");
					add_post_meta($post_id, 'deleted_text', $deleted_text, true);
				}
				if (!empty($replaced_by)) {
					if ($this->debug) echo("<li> topic is replaced by: $replaced_by</li>\n");
					add_post_meta($post_id, 'replaced_by', $replaced_by, true);
				}

            } // end foreach

			if (count($post_updates)) {
				$post_text = "<p>Following an update to the help content. Amongst the topics that have been updated, the following ones have notes associated with them.</p>\n<ul>";
				foreach ($post_updates as $pu) {
					if ($this->debug) echo "<li>".__('Post Updated: ') . $pu['title'] . ". Slug = " . $pu['slug'] . "</li>\n"; flush();
					$post_text .= '<li><a href="' . $pu['slug'] . '">' .	$pu['title'] . "</a></li>\n";
				} // end for each
				$post_text .= "</ul><p>You may wish to check the notes are still relevant.</p>\n";

				$post_author = 1;
				$post_date_gmt = '';
				$post_date = '';
				$post_content = $wpdb-> escape($post_text);
				$post_title = $wpdb-> escape('Updates to help topics');
				$post_name = '';
				$post_path = '';
				$post_excerpt = '';
                $post_parent =  '';
                $post_parent_tab =  '';
				$post_status = 'publish';
				$post_type = 'post';
                $postdata = compact('ID', 'post_author', 'post_date_gmt', 'post_date',
                    'post_content', 'post_title', 'post_name', 'post_path',
                    'post_excerpt', 'post_parent', 'post_parent_tab', 'post_status', 'post_type');

				$post_id = wp_insert_post($postdata);
				if ($this->debug) echo "<li>".__('Inserted topic update post: ') . $post_id . "</li>\n"; flush();

			} // end if significant updates

        } // end if rows
    } // end process_updates


    function sort_parentage() {
		global $wpdb;
        // create map of slugs to post_ids
        // for each entry: if post_parent_name defined, find id and update post_parent with id.

        $query = " SELECT $this->table_name.ID, $this->table_name.post_parent_name , $wpdb->posts.ID as parentId " .
                 " FROM $this->table_name  " .
                 " RIGHT JOIN $wpdb->posts ON $this->table_name.post_parent_name = $wpdb->posts.post_name " .
                 " WHERE $this->table_name.post_parent_name != '' " .
                 " ORDER BY $wpdb->posts.ID";
        $rows = $wpdb-> get_results($query,  ARRAY_A);

		if ($rows != null) {
            foreach ($rows as $row) {
                $post_id = $row['ID'];
                $parent_id = $row['parentId'];
                if ($this->debug) echo("<li>Setting parent of topic $post_id to be $parent_id</li>\n"); flush();


                if (true) {
                    // this is quick but may break in the future
                    $res = $wpdb-> query("UPDATE " . $wpdb->posts . " SET post_parent='$parent_id' WHERE ID=$post_id");
                } else {
                    // this is slow but works by the book
                    $postdata['ID'] = $post_id;
                    $postdata['post_parent'] = $parent_id;
                    wp_update_post($postdata);
                    if ($this->debug) echo("<li>Called update post for id=" . $post_id . "</li>\n"); flush();
                }

			} // end for each
		} // end if any rows

	} // end sort_parentage

    function delete_posts_from_staging_table() {
		global $wpdb;
        $query = "SELECT $wpdb->posts.ID \n" .
                 " FROM $wpdb->posts \n" .
                 " WHERE post_name \n" .
                 "   IN ( \n" .
                 "     SELECT post_name \n" .
                 "     FROM $this->table_name \n" .
                 "     WHERE post_name <> '' \n" .
                 "   ) \n" .
                 "   AND ID NOT \n" .
                 "     IN (  \n" .
                 "       SELECT $wpdb->posts.id \n" .
                 "       FROM $wpdb->posts \n" .
                 "       WHERE post_name \n" .
                 "         IN ( \n" .
                 "           SELECT post_name \n" .
                 "           FROM $this->table_name \n" .
                 "           WHERE post_name <> '' \n" .
                 "         ) \n" .
                 "         AND post_parent NOT \n" .
                 "           IN (  \n" .
                 "             SELECT $wpdb->posts.ID \n" .
                 "             FROM $wpdb->posts \n" .
                 "             WHERE post_name \n" .
                 "               IN ( \n" .
                 "                 SELECT post_name \n" .
                 "                 FROM $this->table_name \n" .
                 "                 WHERE post_name <> '' \n" .
                 "               ) \n" .
                 "           ) \n" .
                 "   ) \n";
        //echo("<li>query=<pre>\n" . $query . "\n</pre></li>\n"); flush();

        $rows = $wpdb-> get_col($query);
        //echo('<li>rows= ' . count($rows) . "  </li>\n"); flush();
        if ($rows != null) {
            $slugs = join(",", $rows);
            if ($this->debug_extra) echo("<li>About to clear out the following slugs='" . $slugs . "'</li>\n"); flush();
            $query = "DELETE FROM " . $wpdb-> posts . " WHERE ID in(" . $slugs . ") ";
            //echo("<li>query=<pre>\n" . $query . "\n</pre></li>\n"); flush();
            $res = $wpdb-> query($query);
			echo('<li>Cleaned up ' . $wpdb-> rows_affected . " posts </li>\n"); flush();
        }
    } // end delete_posts_from_staging_table

	function check_page_exists($pagename) {
		global $wpdb;
		if ($this->debug) echo("<li>check_page_exists: $pagename</li>\n");

		$id = $wpdb-> get_var('SELECT ID FROM ' . $wpdb->posts . " WHERE post_name = '$pagename'");
		if ($this->debug) echo("<li>check_page_exists: got id $id from posts</li>\n");
		if (null == $id) {
			$id = $wpdb-> get_var('SELECT ID FROM ' . $this->table_name . " WHERE post_name = '$pagename'");
			if ($this->debug) echo("<li>check_page_exists: got id $id from staging</li>\n");
		}
		if ($this->debug) echo("<li>check_page_exists: returning id $id</li>\n");
		return $id;
	} // end check_page_exists


	function get_new_posts() {
		global $wpdb;
	  // select from staging table
		// create new posts
		$rows = $wpdb-> get_results("SELECT * FROM " . $this-> table_name . " WHERE is_new = 'Y' AND ID = 0 LIMIT $this->loop_size", ARRAY_A);
        $index = 0;
		if ($rows != null) {
			foreach ($rows as $row) {
                if ($this->debug) echo "<li>".__('Get data for new post ') . $row['post_name'] . "</li>\n";
				$post_author = 1;
				$post_date_gmt = $wpdb-> escape($row['post_date']);
				$post_date = get_date_from_gmt($post_date_gmt);
				$post_content = $wpdb-> escape($row['post_content']);
				$post_title = $wpdb-> escape($row['post_title']);
				$post_name = $wpdb-> escape($row['post_name']);
				$post_path = $wpdb-> escape($row['post_path']);
				$post_excerpt = $wpdb-> escape($row['post_excerpt']);
                $post_parent =  $wpdb-> escape($row['post_parent']);
                $post_parent_tab =  $wpdb-> escape($row['post_parent_tab']);
				$post_prodname = $wpdb-> escape($row['post_prodname']);
				$deleted_text = $wpdb-> escape($row['deleted_text']);
				$replaced_by = $wpdb-> escape($row['replaced_by']);
				$post_status = 'publish';
				$post_type = $this->HELP_IMPORT_POST_TYPE;
				$this-> posts[$index] = compact('post_type', 'post_author', 'post_date', 'post_date_gmt',
												'post_content', 'post_title', 'post_name', 'post_path',
												'post_excerpt', 'post_parent', 'post_parent_tab',
												'post_prodname', 'deleted_text', 'replaced_by',
												'post_status', 'categories');
				$index++;
			}
		} // end if results
        return $index;
	} // end get_new_posts

	function insert_pages($check_first = true) {
		global $wpdb;
		echo '<ol>';

		foreach ($this-> posts as $post) {
			$doit = true;
			extract($post);
			echo "<li>".__('Importing topic ') . $post_name . '...'; flush();

			if ($check_first && $post_id = post_exists($post_title, $post_content, $post_date)) {
			    if ($this->debug_extra) echo __("checking for exists: $post_title, $post_date ");
				_e($post_name .' post already imported. ' . 'Duplicate title?' .  $post_title);
                // update row in staging
				//echo(" UPDATE " . $this-> table_name . " SET ID = " . -1 . " WHERE post_name like '" . $post_name . "'");
                $res = $wpdb-> query("UPDATE " . $this-> table_name . " SET is_new='N' WHERE post_name like '" . $post_name . "'");
			} else {
				$post_id = wp_insert_post($post);
				if (!$post_id) {
					_e("Couldn't get post ID");
					return;
				} else {
					// update row in staging
					$res = $wpdb-> query("UPDATE " . $this-> table_name . " SET ID=" . intval($post_id) . " WHERE post_name='" . $post_name . "'");
				}
                //echo(" prodname is $post_prodname, ");
                //split prodname on colon (format is abbrev:long name)
                list($prodname_id, $prodname_long) = split(':', $post_prodname);
                add_post_meta($post_id, 'prodname_id', $prodname_id, true);
                add_post_meta($post_id, 'prodname_long', $prodname_long, true);
				if (!empty($deleted_text)) {
					if ($this->debug) echo("<li> topic is deleted: $deleted_text</li>\n");
					add_post_meta($post_id, 'deleted_text', $deleted_text, true);
				}
				if (!empty($replaced_by)) {
					if ($this->debug) echo("<li> topic is replaced by: $replaced_by</li>\n");
					add_post_meta($post_id, 'replaced_by', $replaced_by, true);
				}
                if (false) { // not yet
                    if (0 != count($categories)) {
                        $ids = wp_create_categories($categories, $post_id);
                        echo "cat id = " . $ids[0];
                    }
                }
				_e(' Done !' . " new id = $post_id"); flush();

			} // end else
			echo '</li>';flush();
		}

		echo '</ol>';
	} // end insert_pages


	function update_guids() {
		global $wpdb;
		$rows = $wpdb-> get_results("SELECT * FROM " . $wpdb->posts, ARRAY_A);
        if ($rows != null) {
            foreach ($rows as $row) {
                if ($this->debug) echo("<li> Updating guid to " . get_permalink($row['ID']) . "</li>\n");
                $wpdb->query("UPDATE $wpdb->posts SET guid = '" . get_permalink($row['ID']) . "' WHERE ID = '" . $row['ID'] . "'");
            }
        }
	} // end update_guids




    // database table create / update code
	function __construct() {
		global $wpdb;
        if (function_exists( 'is_site_admin' ) ) {  // are we on MU?
            $this-> is_wpmu = true;
        }

        $home = get_option( 'home' );
        //echo '<p>Home is ' . $home . '</p>';
        if ( $home != '' ) {
            $home_path = parse_url( $home );
            $home_path = $home_path['path'];
        }
        //echo '<p>Home path is ' . $home_path . '</p>';
        $this->HELP_ROOT_PATH = $home_path;


		if ( $wpdb->supports_collation() ) {
			if ( ! empty($wpdb->charset) )
				$charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
			if ( ! empty($wpdb->collate) )
				$charset_collate .= " COLLATE $wpdb->collate";
		}

		// set up vars
		$this-> table_name = $wpdb-> prefix . 'help_page_staging';
		$this-> table_def = "CREATE TABLE `" . $this-> table_name . "` (" .
			" `ID` bigint(20) unsigned NOT NULL default '0'," .
			" `post_author` bigint(20) NOT NULL default '0'," .
			" `post_date` datetime NOT NULL default '0000-00-00 00:00:00'," .
			" `post_content` longtext NOT NULL," .
			" `post_title` text  NOT NULL," .
			" `post_category` int(4) NOT NULL default '0'," .
			" `post_excerpt` text NOT NULL, " .
			" `post_keywords` text," .
			" `post_name` varchar(200) NOT NULL default ''," .
			" `post_parent` bigint(20) NOT NULL default '0'," .
			" `post_parent_name` varchar(200) NOT NULL default ''," .
			" `post_parent_tab` varchar(200) NOT NULL default ''," .
			" `post_prodname` varchar(255) NOT NULL default ''," .
			" `deleted_text` text NOT NULL default '', " .
			" `replaced_by` varchar(200) NOT NULL default '', " .
			" `post_path` text NOT NULL default ''," .
			" `menu_order` int(11) NOT NULL default '0'," .
			" `is_path` enum('Y','N') NOT NULL default 'N'," .
			" `is_deleted` enum('Y','N') NOT NULL default 'N'," .
			" `has_comments` enum('Y','N') NOT NULL default 'N'," .
			" `is_updated` enum('Y','N','U') NOT NULL default 'N'," .
			" `is_new` enum('Y','N') NOT NULL default 'N'," .
			" UNIQUE `post_name` (`post_name`)" .
			" ) $charset_collate ";
	} // end __construct

	function update() {
		global $wpdb;
		$now_version  = intval(get_option($this-> HELP_IMPORT_VERSION));
		while ($now_version < intval($this-> version)) {
			switch($now_version) {
				case 91: // update to 92: add excerpt column
					if ($this->debug) echo("<li> Updating table. Adding excerpt column </li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `post_excerpt` text NOT NULL ");
					$now_version++;
					break;
				case 92: // update to 93: add post_parent_name column
					if ($this->debug) echo("<li> Updating table. Adding post_parent_name column </li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `post_parent_name` varchar(200) NOT NULL default '' ");
					$now_version++;
					break;
				case 93: // update to 94: add post_parent_tab column
					if ($this->debug) echo("<li> Updating table. Adding post_parent_tab column </li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `post_parent_tab` varchar(200) NOT NULL default '' ");
					$now_version++;
					break;
				case 94: // update to 95: add post_prodname column
					if ($this->debug) echo("<li> Updating table. Adding post_prodname column </li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `post_prodname` varchar(255) NOT NULL default '' ");
					$now_version++;
					break;
				case 95: // update to 96: add new value to is_updated
					if ($this->debug) echo("<li> Updating table. Adding new updated value </li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " MODIFY COLUMN `is_updated` enum('Y','N','U') NOT NULL default 'N' ");
					$now_version++;
					break;
				case 96: // update to 97: add deleted and replacedby
					if ($this->debug) echo("<li> Updating table. Adding deleted_text and replaced_by coliumns</li>\n");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `deleted_text` text NOT NULL default '' ");
					$wpdb-> query("ALTER TABLE `" . $this-> table_name . "` " . " ADD COLUMN `replaced_by` varchar(200) NOT NULL default '' ");
					$now_version++;
					break;
				default:
					echo('<li class="error"><strong>Woah! Unknown old version!</strong></li>' . "\n");
					$now_version = $this-> version;
					break;
			} // end switch
		}
		return $now_version;
	} // end update

	function Ditahelp_Import() {
	} // end Ditahelp_Import

}

$ditahelp_import = new Ditahelp_Import();

register_importer('ditahelp', __('DITA help'), __('Import pages from a DITA generated help XHTML hierarchy'), array ($ditahelp_import, 'dispatch'));


// ==================
// An implementation of PHP's realpath() (from ilgarm -at- azeronline.com via
// http://uk3.php.net/realpath) that works with non-existant directories that
// should be resolved out.
define( "_PL_OS_SEP", "/" );
define( "_CUR_OS", substr( php_uname( ), 0, 7 ) == "Windows" ? "Win" : "_Nix" );

function checkCurrentOS( $_OS )
{
    if ( strcmp( $_OS, _CUR_OS ) == 0 ) {
        return true;
    }
    return false;
}

function isRelative( $_dir )
{
    if ( checkCurrentOS( "Win" ) ) {
        return ( preg_match( "/^\w+:/", $_dir ) <= 0 );
    }
    else {
        return ( preg_match( "/^\//", $_dir ) <= 0 );
    }
}

function unifyPath( $_path )
{
    if ( checkCurrentOS( "Win" ) ) {
        return str_replace( "\\", _PL_OS_SEP, $_path );
    }
    return $_path;
}

function getRealpath( $_path )
{
    /*
     * This is the starting point of the system root.
     * Left empty for UNIX based and Mac.
     * For Windows this is drive letter and semicolon.
     */
    $__path = $_path;
    if ( isRelative( $_path ) ) {
        $__curdir = unifyPath( realpath( "." ) . _PL_OS_SEP );
        $__path = $__curdir . $__path;
    }
    $__startPoint = "";
    if ( checkCurrentOS( "Win" ) ) {
        list( $__startPoint, $__path ) = explode( ":", $__path, 2 );
        $__startPoint .= ":";
    }
    # From now processing is the same for WIndows and Unix, and hopefully for others.
    $__realparts = array( );
    $__parts = explode( _PL_OS_SEP, $__path );
    for ( $i = 0; $i < count( $__parts ); $i++ ) {
        if ( strlen( $__parts[ $i ] ) == 0 || $__parts[ $i ] == "." ) {
            continue;
        }
        if ( $__parts[ $i ] == ".." ) {
            if ( count( $__realparts ) > 0 ) {
                array_pop( $__realparts );
            }
        }
        else {
            array_push( $__realparts, $__parts[ $i ] );
        }
    }
    return $__startPoint . _PL_OS_SEP . implode( _PL_OS_SEP, $__realparts );
}

//echo "getRealpath ../../x: ". getRealpath( "../../x" ) . "<BR>\n";
//echo "getRealpath ./../x: ". getRealpath( "./../x" ) . "<BR>\n";
//echo "getRealpath x: ". getRealpath( "x" ) . "<BR>\n";
//echo "getRealpath /../x: ". getRealpath( "/../x" ) . "<BR>\n";
//echo "getRealpath d:/../../x: ". getRealpath( "d:/../../x" ) . "<BR>\n";
//echo "getRealpath ./../xx/xxx/.////../yyy: ". getRealpath( "./../xx/xxx/.////../yyy" ) . "<BR>\n";


function mkdirs($dir, $mode = 0777)
{
	if (is_dir($dir) || @mkdir($dir,$mode)) return TRUE;
	if (!mkdirs(dirname($dir),$mode)) return FALSE; // see recursion
	return @mkdir($dir,$mode);
}

function _dhcmp($a, $b) {
	if ($a == $b) {
		return 0;
	}
	$namea = get_blog_option( $a['blog_id'], "blogname" );
	$nameb = get_blog_option( $b['blog_id'], "blogname" );
	//error_log("ditahelp comparing $namea with $nameb");
	return strcasecmp($namea, $nameb);
}

?>
